@@ -27,6 +27,7 @@ tag_attrib = '_yaml_tag'
 class Comment(object):
     # sys.getsize tested the Comment objects, __slots__ makes them bigger
     # and adding self.end did not matter
+    __slots__ = 'comment', '_items', '_end', '_start',
     attrib = comment_attrib
 
     def __init__(self):
@@ -73,6 +74,7 @@ def NoComment():
 
 
 class Format(object):
+    __slots__ = '_flow_style',
     attrib = format_attrib
 
     def __init__(self):
@@ -131,6 +133,7 @@ class LineCol(object):
 
 
 class Anchor(object):
+    __slots__ = 'value', 'always_dump',
     attrib = anchor_attrib
 
     def __init__(self):
@@ -140,6 +143,7 @@ class Anchor(object):
 
 class Tag(object):
     """store tag information for roundtripping"""
+    __slots__ = 'value',
     attrib = tag_attrib
 
     def __init__(self):
@@ -154,7 +158,9 @@ class CommentedBase(object):
         return getattr(self, Comment.attrib)
 
     def yaml_end_comment_extend(self, comment, clear=False):
-        if clear:
+        if comment is None:
+            return
+        if clear or self.ca.end is None:
             self.ca.end = []
         self.ca.end.extend(comment)
 
@@ -180,17 +186,47 @@ class CommentedBase(object):
 
     def yaml_set_start_comment(self, comment, indent=0):
         """overwrites any preceding comment lines on an object
-        expects comment to be without `#` and possible have mutlple lines
+        expects comment to be without `#` and possible have multiple lines
         """
-        from .error import Mark
+        from .error import CommentMark
         from .tokens import CommentToken
         pre_comments = self._yaml_get_pre_comment()
         if comment[-1] == '\n':
             comment = comment[:-1]  # strip final newline if there
-        start_mark = Mark(None, None, None, indent, None, None)
+        start_mark = CommentMark(indent)
         for com in comment.split('\n'):
             pre_comments.append(CommentToken('# ' + com + '\n', start_mark, None))
 
+    def yaml_set_comment_before_after_key(self, key, before=None, indent=0,
+                                          after=None, after_indent=None):
+        """
+        expects comment (before/after) to be without `#` and possible have multiple lines
+        """
+        from ruamel.yaml.error import CommentMark
+        from ruamel.yaml.tokens import CommentToken
+
+        def comment_token(s, mark):
+            # handle empty lines as having no comment
+            return CommentToken(('# ' if s else '') + s + '\n', mark, None)
+
+        if after_indent is None:
+            after_indent = indent + 2
+        if before and before[-1] == '\n':
+            before = before[:-1]  # strip final newline if there
+        if after and after[-1] == '\n':
+            after = after[:-1]  # strip final newline if there
+        start_mark = CommentMark(indent)
+        c = self.ca.items.setdefault(key, [None, [], None, None])
+        if before:
+            for com in before.split('\n'):
+                c[1].append(comment_token(com, start_mark))
+        if after:
+            start_mark = CommentMark(after_indent)
+            if c[3] is None:
+                c[3] = []
+            for com in after.split('\n'):
+                c[3].append(comment_token(com, start_mark))
+
     @property
     def fa(self):
         """format attribute
@@ -207,7 +243,7 @@ class CommentedBase(object):
         the #. The column index is for the # mark
         """
         from .tokens import CommentToken
-        from .error import Mark
+        from .error import CommentMark
         if column is None:
             column = self._yaml_get_column(key)
         if comment[0] != '#':
@@ -216,7 +252,7 @@ class CommentedBase(object):
             if comment[0] == '#':
                 comment = ' ' + comment
                 column = 0
-        start_mark = Mark(None, None, None, column, None, None)
+        start_mark = CommentMark(column)
         ct = [CommentToken(comment, start_mark, None), None]
         self._yaml_add_eol_comment(ct, key=key)
 
@@ -262,7 +298,7 @@ class CommentedBase(object):
 
 
 class CommentedSeq(list, CommentedBase):
-    __slots__ = [Comment.attrib, ]
+    __slots__ = Comment.attrib,
 
     def _yaml_add_comment(self, comment, key=NoComment):
         if key is not NoComment:
@@ -391,7 +427,6 @@ class CommentedMapView(Sized):
 
 
 class CommentedMapKeysView(CommentedMapView, Set):
-
     __slots__ = ()
 
     @classmethod
@@ -408,7 +443,6 @@ class CommentedMapKeysView(CommentedMapView, Set):
 
 
 class CommentedMapItemsView(CommentedMapView, Set):
-
     __slots__ = ()
 
     @classmethod
@@ -430,7 +464,6 @@ class CommentedMapItemsView(CommentedMapView, Set):
 
 
 class CommentedMapValuesView(CommentedMapView):
-
     __slots__ = ()
 
     def __contains__(self, value):
@@ -445,7 +478,7 @@ class CommentedMapValuesView(CommentedMapView):
 
 
 class CommentedMap(ordereddict, CommentedBase):
-    __slots__ = [Comment.attrib, ]
+    __slots__ = Comment.attrib,
 
     def _yaml_add_comment(self, comment, key=NoComment, value=NoComment):
         """values is set to key to indicate a value attachment of comment"""
@@ -700,11 +733,11 @@ class CommentedMap(ordereddict, CommentedBase):
 
 
 class CommentedOrderedMap(CommentedMap):
-    __slots__ = [Comment.attrib, ]
+    __slots__ = Comment.attrib,
 
 
 class CommentedSet(MutableSet, CommentedMap):
-    __slots__ = [Comment.attrib, 'odict']
+    __slots__ = Comment.attrib, 'odict',
 
     def __init__(self, values=None):
         self.odict = ordereddict()
